﻿using DocuSignAPI.APIService;
using System;
using System.Collections.Generic;
using System.Linq;


namespace EnvelopeCreatorProject.Models.Recipients
{
    internal class SignerRecipients:RecipientType
    {
        private readonly List<Tab> _signHereTabs;
        private readonly List<Tab> _initialsTabs;
        private readonly List<Tab> _dataSignFields;

        internal SignerRecipients(int totalCount, List<Tab> signHereTabs, List<Tab> initialsTabs, List<Tab> dataSignFields)
            : base(totalCount)
        {
            _signHereTabs = signHereTabs;
            _initialsTabs = initialsTabs;
            _dataSignFields = dataSignFields;        
        }

        protected override void AddData(Recipient objRecipient){}

        protected bool AddTabs(List<Tab> tabs, Recipient objRecipient, string name, bool isFirstBlock)
        {
            string recipientID = objRecipient.ID;
            List<Tab> recipientTabs = tabs.FindAll((Tab tab) => tab.RecipientID == recipientID);
            if (!recipientTabs.Any()) return false;
            if (!isFirstBlock) _recipients.AppendLine(",");
            _recipients.AppendLine("\"" + name + "\":[");
            _recipients.AppendLine(String.Join(",", recipientTabs.Select(getTabMetaData)));
            _recipients.AppendLine("]");
            return recipientTabs.Any();
        }

        protected string getTabMetaData(Tab tab)
        {
            return "{" +
            "\"xPosition\":\"" + tab.XPosition + "\"," +
            "\"yPosition\":\"" + tab.YPosition + "\"," +
            "\"documentId\":\"" + tab.DocumentID + "\"," +
            "\"pageNumber\":\"" + tab.PageNumber + "\"," +
            "\"recipientId\":\"" + tab.RecipientID + "\"," +
            "}";
        }

        protected void AddSignatureInfoTo(Recipient objRecipient)
        {
            _recipients.AppendLine(", ");
            if (objRecipient.SignatureInfo == null)
                return;

            _recipients.AppendLine("\"signatureInfo\":{");
            _recipients.AppendLine("\"fontStyle\":\"" + objRecipient.SignatureInfo.FontStyle.ToString() + "\",");
            _recipients.AppendLine("\"signatureName\":\"" + objRecipient.SignatureInfo.SignatureName + "\",");
            _recipients.AppendLine("\"signatureInitials\":\"" + objRecipient.SignatureInfo.SignatureInitials + "\"");
            _recipients.AppendLine("},");
        }

        protected void AddSignTabs(Recipient objRecipient)
        {
            _recipients.AppendLine("\"tabs\":{");

            //First, add SignHere Tabs (FullName as signature)
            AddTabs(_signHereTabs, objRecipient, "signHereTabs", true);

            //Now add SignInitials Tabs
            AddTabs(_initialsTabs, objRecipient, "initialHereTabs", false);

            //Finally, add Date Signed Tabs
            AddTabs(_dataSignFields, objRecipient, "dateSignedTabs", false);
            _recipients.AppendLine("}");
        }
    }
}
